package net.wanji.utc.hisense.service.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.common.enums.*;
import net.wanji.utc.hisense.pojo.CrossPhaseDirTurnPojo;
import net.wanji.utc.hisense.pojo.convert.HisenseLightStatusPojo;
import net.wanji.utc.hisense.service.SignalStatusService;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:28
 */
@Service
@Slf4j
public class SignalStatusServiceImpl implements SignalStatusService {

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) {
        return null;
    }

    @Override
    public List<LightsStatusVO> lightStatus(String crossId) {
        List<LightsStatusVO> lightsStatusVOS = new ArrayList<>();
        LightsStatusVO lightsStatusVO = SignalDataCache.runningStateInfoCache4DataBrain.get(crossId);
        lightsStatusVOS.add(lightsStatusVO);
        return lightsStatusVOS;
    }

    @Override
    public void receiveLightStatus(HisenseLightStatusPojo hisenseLightStatusPojo) throws Exception {
        ObjectMapper jackson = JacksonUtils.getInstance();
        String bodyStr = hisenseLightStatusPojo.getBody();
        List<HisenseLightStatusPojo.Body> bodies = jackson.readValue(bodyStr, new TypeReference<List<HisenseLightStatusPojo.Body>>() {});
        if (!CollectionUtils.isEmpty(bodies)) {
            for (HisenseLightStatusPojo.Body body : bodies) {
                HisenseLightStatusPojo.ContentBody content = body.getContent();
                LightsStatusVO lightsStatusVO = new LightsStatusVO();
                String hiCrossId = content.getCrossId();
                CrossInfoPO crossInfo = CrossInfoCache.getCrossInfoBySignalCode(hiCrossId);
                Long signalMachineStamp = content.getSignalMachineStamp();
                if (Objects.isNull(crossInfo)) {
                    continue;
                }
                String crossId = crossInfo.getId();
                Long lastSignalMachineStamp = SignalDataCache.runningStatusStampMap.get(crossId);
                if (Objects.isNull(lastSignalMachineStamp)) {
                    SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                } else {
                    if (lastSignalMachineStamp >= signalMachineStamp) {
                        continue;
                    }
                    SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                }
                int cycleCountDown = content.getCycleCountDown();
                int hiControlMode = content.getControlMode();
                int period = content.getPeriod();
                int planId = (content.getPlanId() + 2) / 3;
                lightsStatusVO.setCrossId(crossId);
                lightsStatusVO.setSchemeId(String.valueOf(planId));
                String runMode = String.valueOf(DataBrainControlModeEnum.getWjControlMode(hiControlMode));
                lightsStatusVO.setRunMode(runMode);
                lightsStatusVO.setCycleCountDown(cycleCountDown);
                lightsStatusVO.setCycleLen(period);
                lightsStatusVO.setSchemeStartTime(String.valueOf(signalMachineStamp));
                lightsStatusVO.setPhaseMap(Collections.emptyMap());
                lightsStatusVO.setControlMode(runMode);
                lightsStatusVO.setCode(hiCrossId);
                lightsStatusVO.setManufacturerCode("Hisense");
                lightsStatusVO.setPhasePlanId(String.valueOf(planId));

                List<HisenseLightStatusPojo.ContentBody.LampRealInfos> lampRealInfos = content.getLampRealInfos();
                if (!CollectionUtils.isEmpty(lampRealInfos)) {
                    Map<Integer, List<HisenseLightStatusPojo.ContentBody.LampRealInfos>> dirLampMap = lampRealInfos.stream().collect(Collectors.groupingBy(HisenseLightStatusPojo.ContentBody.LampRealInfos::getRoadDirection));
                    Map<String, Object> dirMap = new HashMap<>();
                    Map<Integer, List<Integer>> dirTurnMap = new HashMap<>();
                    for (Map.Entry<Integer, List<HisenseLightStatusPojo.ContentBody.LampRealInfos>> dirEntry : dirLampMap.entrySet()) {
                        Integer roadDirection = DataBrainDirEnum.getWjLightsDir(dirEntry.getKey());
                        List<HisenseLightStatusPojo.ContentBody.LampRealInfos> dirLampList = dirEntry.getValue();
                        Map<String, String> turnMap = new HashMap<>();
                        List<Integer> hisTurnList = new ArrayList<>();
                        for (HisenseLightStatusPojo.ContentBody.LampRealInfos lampRealInfo : dirLampList) {
                            String lampStatus = lampRealInfo.getLampStatus();
                            Integer countDown = lampRealInfo.getCountDown();
                            Integer roadFlowDirection = lampRealInfo.getRoadFlowDirection();
                            Integer wjTurn = DataBrainTurnEnum.getWjTurn(roadFlowDirection);
                            if (StringUtils.endsWithIgnoreCase(lampStatus, "23")) {
                                lightsStatusVO.setCyclePhaseCountDown(countDown);
                                lightsStatusVO.setCycleCountDown(countDown);
                                hisTurnList.add(wjTurn);
                            }
                            turnMap.put(String.valueOf(wjTurn), DataBrainLampStatusEnum.getWjLampStatus(lampStatus));
                        }
                        dirMap.put(String.valueOf(roadDirection), turnMap);
                        // 获取当前绿灯方向转向
                        if (!CollectionUtils.isEmpty(hisTurnList)) {
                            dirTurnMap.put(roadDirection, hisTurnList);
                        }
                    }

                    // 通过路口编号获取当前相位缓存
                    Integer currentPhaseNo = getCurrentPhagetseNo(crossId, planId, dirTurnMap);
                    lightsStatusVO.setPhaseId(String.valueOf(currentPhaseNo));
                    lightsStatusVO.setDirLampGroupMap(dirMap);
                    SignalDataCache.runningStateInfoCache4DataBrain.put(crossId, lightsStatusVO);
                }
            }
        }
    }

    /**
     * @description 通过方向转向获取路口方案的相位号
     */
    private static Integer getCurrentPhagetseNo(String crossId, int planId, Map<Integer, List<Integer>> hisDirTurnMap) {
        Integer phaseNo = null;
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnList = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        if (!CollectionUtils.isEmpty(crossPhaseDirTurnList)) {
            List<CrossPhaseDirTurnPojo> schemeNoList = crossPhaseDirTurnList.stream().filter(po -> Objects.equals(po.getSchemeNo(), planId)).collect(Collectors.toList());
            for (CrossPhaseDirTurnPojo crossPhaseDirTurnPojo : schemeNoList) {
                Map<Integer, List<Integer>> dirTurnCache = crossPhaseDirTurnPojo.getDirTurnMap();
                for (Map.Entry<Integer, List<Integer>> dirTurnCacheEntry : dirTurnCache.entrySet()) {
                    Integer dirCache = dirTurnCacheEntry.getKey();
                    for (Map.Entry<Integer, List<Integer>> hisEntry : hisDirTurnMap.entrySet()) {
                        Integer hisDir = hisEntry.getKey();
                        if (Objects.equals(dirCache, CrossLisghtsLaneDirEnum.getLightsDirByLaneDir(hisDir))) {
                            List<Integer> turnListCache = dirTurnCacheEntry.getValue();
                            if (hisEntry.getValue().stream().allMatch(turnListCache::contains)) {
                                phaseNo = crossPhaseDirTurnPojo.getPhaseNo();
                            }
                        }
                    }
                }
            }
        }
        log.error("当前路口：{},方案号：{}，相位号：{}", crossId, planId, phaseNo);
        return phaseNo;
    }


}
