package net.wanji.utc.hisense.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossPhasePO;
import net.wanji.databus.dao.entity.CrossSchemePO;
import net.wanji.databus.dao.mapper.CrossPhaseMapper;
import net.wanji.databus.dao.mapper.CrossSchemeMapper;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.cache.SignalDataCache;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/09/20 14:25
 */
@Component
@Slf4j
public class HisensePhaseCountDownTask {

    @Resource
    private CrossPhaseMapper crossPhaseMapper;
    @Resource
    private CrossSchemeMapper crossSchemeMapper;

    @Scheduled(fixedRate = 1000, initialDelay = 1000)
    public void phaseTimeCountDown() {
        Map<String, LightsStatusVO> runningStateInfoCache4DataBrain = SignalDataCache.runningStateInfoCache4DataBrain;
        Map<String, Long> runningStatusStampMap = SignalDataCache.runningStatusStampMap;
        if (!runningStateInfoCache4DataBrain.isEmpty()) {
            for (Map.Entry<String, LightsStatusVO> entry : runningStateInfoCache4DataBrain.entrySet()) {
                String crossId = entry.getKey();
                LightsStatusVO lightsStatusVO = entry.getValue();
                String schemeStartTime = lightsStatusVO.getSchemeStartTime();
                Long lastPhaseTimeStamp = runningStatusStampMap.get(crossId);
                String phaseId = lightsStatusVO.getPhaseId();
                String schemeId = lightsStatusVO.getSchemeId();
//                log.error("当前相位号：{}，上一时间戳：{}，当前时间时间戳：{}", crossId, lastPhaseTimeStamp, schemeStartTime);
//                log.error("当前路口参数：{}", entry);
                if (Objects.nonNull(lastPhaseTimeStamp) && StringUtils.equalsIgnoreCase(schemeStartTime, String.valueOf(lastPhaseTimeStamp))) {
                    CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, Integer.valueOf(schemeId));
                    if (lightsStatusVO.getCyclePhaseCountDown() == 0 && Objects.nonNull(crossSchemePO)) {
                        List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(crossSchemePO.getId()));
                        Integer nextPhaseTime = 0;
                        String nextPhaseId = "0";
                        if (!CollectionUtils.isEmpty(crossPhasePOS)) {
                            for (int i = 0; i < crossPhasePOS.size(); i++) {
                                if (StringUtils.equalsIgnoreCase(phaseId, crossPhasePOS.get(i).getPhaseNo())) {
                                    if (i == crossPhasePOS.size() - 1) {
                                        nextPhaseTime = crossPhasePOS.get(0).getPhaseTime();
                                        nextPhaseId = "1";
                                    } else {
                                        nextPhaseTime = crossPhasePOS.get(i + 1).getPhaseTime();
                                        nextPhaseId = crossPhasePOS.get(i + 1).getPhaseNo();
                                    }
                                }
                            }
                        }
                        lightsStatusVO.setPhaseId(nextPhaseId);
                        lightsStatusVO.setCyclePhaseCountDown(nextPhaseTime);
                    } else {
                        lightsStatusVO.setCyclePhaseCountDown(lightsStatusVO.getCyclePhaseCountDown() - 1);
                    }
                    runningStateInfoCache4DataBrain.put(crossId, lightsStatusVO);
                }
            }
        }

    }
}
